#ifndef MSP_CORE_MAPUTILS_H_
#define MSP_CORE_MAPUTILS_H_

#include <stdexcept>
#include <typeinfo>
#include <msp/strings/lexicalcast.h>

namespace Msp {

namespace MapUtilsInternal {

/* This must be hidden in the internal namespace to avoid interfering with
other things.  There may be problems if a key type has operator<< for ostream
but not LexicalConverter. */
template<typename T>
void operator<<(LexicalConverter &, const T &)
{ }

template<typename T>
static std::string stringify_key(const T &k)
{
	try
	{
		LexicalConverter conv((Fmt()));
		conv<<k;
		return conv.get();
	}
	catch(const lexical_error &)
	{
		return "<cannot display value>";
	}
}

} // namespace Internal

class key_error: public std::runtime_error
{
public:
	template<typename T>
	key_error(const T &k):
		runtime_error(make_what(typeid(T), MapUtilsInternal::stringify_key(k)))
	{ }

	virtual ~key_error() throw() { }

private:
	static std::string make_what(const std::type_info &, const std::string &);
};


template<typename T>
typename T::mapped_type &get_item(T &map, const typename T::key_type &key)
{
	typename T::iterator i = map.find(key);
	if(i==map.end())
		throw key_error(key);

	return i->second;
}

template<typename T>
const typename T::mapped_type &get_item(const T &map, const typename T::key_type &key)
{
	typename T::const_iterator i = map.find(key);
	if(i==map.end())
		throw key_error(key);

	return i->second;
}

template<typename T>
const typename T::iterator insert_unique(T &map, const typename T::key_type &key, const typename T::mapped_type &item)
{
	if(map.count(key))
		throw key_error(key);

	return map.insert(typename T::value_type(key, item)).first;
}

} // namespace Msp

#endif
